"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Plot = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const d3_scale_1 = require("d3-scale");
const d3_scale_chromatic_1 = require("d3-scale-chromatic");
const immer_1 = require("immer");
const react_1 = require("react");
const react_d3_utils_1 = require("react-d3-utils");
const bboxContext_1 = require("../contexts/bboxContext");
const legendContext_1 = require("../contexts/legendContext");
const legendOffsetContext_1 = require("../contexts/legendOffsetContext");
const plotContext_1 = require("../contexts/plotContext");
const plotControllerContext_1 = require("../contexts/plotController/plotControllerContext");
const utils_1 = require("../utils");
const splitChildren_1 = require("../utils/splitChildren");
const usePlotSizes_1 = require("../utils/usePlotSizes");
const Tracking_1 = __importDefault(require("./Tracking"));
const TransparentRect_1 = __importDefault(require("./TransparentRect"));
const reducerCurr = (0, immer_1.produce)(plotContext_1.plotReducer);
const initialState = {
    headingPosition: null,
    legendPosition: null,
    legendMargin: 0,
    series: [],
    axes: {},
};
const defaultSvgStyle = {
    overflow: 'visible',
    fontFamily: 'Arial, Helvetica, sans-serif',
    userSelect: 'none',
    // Without this, Safari on iOS still triggers selection from a long press.
    WebkitUserSelect: 'none',
};
const defaultEventSvgStyle = {
    // Prevent default touch behavior on the plot only when events are enabled.
    touchAction: 'none',
};
function Plot(props) {
    const { width, height, colorScheme = d3_scale_chromatic_1.schemeSet1, margin = {}, svgStyle = {}, svgId, svgClassName, plotViewportStyle = {}, seriesViewportStyle = {}, controllerId, children, } = props;
    const plotId = (0, utils_1.useId)(undefined, 'plot');
    const [state, dispatch] = (0, react_1.useReducer)(reducerCurr, initialState, undefined);
    const { series, annotations, topAxis, rightAxis, bottomAxis, leftAxis, heading, legend, } = (0, splitChildren_1.splitChildren)(children);
    if (width === undefined) {
        throw new Error('width is mandatory');
    }
    if (height === undefined) {
        throw new Error('height is mandatory');
    }
    const plotEvents = (0, plotControllerContext_1.usePlotEventsPlotContext)({ controllerId });
    (0, react_1.useEffect)(() => {
        if (!plotEvents)
            return;
        plotEvents.registerPlot(plotId);
        return () => plotEvents.unregisterPlot(plotId);
    }, [plotEvents, plotId]);
    const plotOverrides = (0, plotControllerContext_1.usePlotOverrides)({ controllerId });
    // Bounding boxes used to adapt viewport size.
    const headingBbox = (0, react_d3_utils_1.useBBoxObserver)();
    const topAxisBbox = (0, react_d3_utils_1.useBBoxObserver)();
    const rightAxisBbox = (0, react_d3_utils_1.useBBoxObserver)();
    const bottomAxisBbox = (0, react_d3_utils_1.useBBoxObserver)();
    const leftAxisBbox = (0, react_d3_utils_1.useBBoxObserver)();
    const legendBbox = (0, react_d3_utils_1.useBBoxObserver)();
    // Distances in plot.
    const { plotWidth, plotHeight, topOffset, leftOffset, legendOffset } = (0, usePlotSizes_1.usePlotSizes)({
        width,
        height,
        margin,
        axes: state.axes,
        topAxisHeight: topAxisBbox.height,
        rightAxisWidth: rightAxisBbox.width,
        bottomAxisHeight: bottomAxisBbox.height,
        leftAxisWidth: leftAxisBbox.width,
        headingPosition: state.headingPosition,
        headingHeight: headingBbox.height,
        legendPosition: state.legendPosition,
        legendMargin: state.legendMargin,
        legendWidth: legendBbox.width,
        legendHeight: legendBbox.height,
    });
    // Set scales.
    const axisContext = (0, plotContext_1.useAxisContext)(state, plotOverrides.axes, {
        plotWidth,
        plotHeight,
    });
    const ids = (0, react_1.useMemo)(() => state.series.map(({ id }) => id), [state.series]);
    const colorScaler = (0, react_1.useMemo)(() => (0, d3_scale_1.scaleOrdinal)().range(colorScheme).domain(ids), [colorScheme, ids]);
    const plotContextValue = (0, react_1.useMemo)(() => {
        return {
            width,
            height,
            plotWidth,
            plotHeight,
            axisContext,
            colorScaler,
        };
    }, [width, height, plotWidth, plotHeight, axisContext, colorScaler]);
    const finalSvgStyle = {
        ...defaultSvgStyle,
        ...(plotEvents ? defaultEventSvgStyle : null),
        ...svgStyle,
    };
    return ((0, jsx_runtime_1.jsx)(plotContext_1.plotContext.Provider, { value: plotContextValue, children: (0, jsx_runtime_1.jsx)(plotContext_1.plotDispatchContext.Provider, { value: dispatch, children: (0, jsx_runtime_1.jsx)(legendContext_1.LegendProvider, { children: (0, jsx_runtime_1.jsxs)("div", { style: { position: 'relative', width: width, height: height }, children: [(0, jsx_runtime_1.jsxs)("svg", { xmlns: "http://www.w3.org/2000/svg", width: width, height: height, style: {
                                ...finalSvgStyle,
                                position: 'absolute',
                                top: '0',
                                left: '0',
                            }, id: svgId, className: svgClassName, children: [(0, jsx_runtime_1.jsx)(TransparentRect_1.default, { width: width, height: height, style: plotViewportStyle }), (0, jsx_runtime_1.jsxs)("g", { transform: `translate(${leftOffset}, ${topOffset})`, children: [(0, jsx_runtime_1.jsx)(TransparentRect_1.default, { width: plotWidth, height: plotHeight, style: seriesViewportStyle }), (0, jsx_runtime_1.jsx)("clipPath", { id: `seriesViewportClip-${plotId}`, children: (0, jsx_runtime_1.jsx)("rect", { width: plotWidth, height: plotHeight }) }), (0, jsx_runtime_1.jsx)("g", { style: { clipPath: `url(#seriesViewportClip-${plotId})` }, children: series }), (0, jsx_runtime_1.jsx)(bboxContext_1.bboxContext.Provider, { value: topAxisBbox.ref, children: (0, jsx_runtime_1.jsx)("g", { children: topAxis }) }), (0, jsx_runtime_1.jsx)(bboxContext_1.bboxContext.Provider, { value: rightAxisBbox.ref, children: (0, jsx_runtime_1.jsx)("g", { children: rightAxis }) }), (0, jsx_runtime_1.jsx)(bboxContext_1.bboxContext.Provider, { value: bottomAxisBbox.ref, children: (0, jsx_runtime_1.jsx)("g", { children: bottomAxis }) }), (0, jsx_runtime_1.jsx)(bboxContext_1.bboxContext.Provider, { value: leftAxisBbox.ref, children: (0, jsx_runtime_1.jsx)("g", { children: leftAxis }) })] }), (0, jsx_runtime_1.jsx)("g", { ref: headingBbox.ref, children: heading })] }), (0, jsx_runtime_1.jsxs)("svg", { xmlns: "http://www.w3.org/2000/svg", width: width, height: height, style: {
                                ...finalSvgStyle,
                                position: 'absolute',
                                top: '0',
                                left: '0',
                            }, id: svgId ? `${svgId}-annotations` : undefined, className: svgClassName, children: [(0, jsx_runtime_1.jsxs)("g", { transform: `translate(${leftOffset}, ${topOffset})`, children: [(0, jsx_runtime_1.jsx)("g", { style: { clipPath: `url(#seriesViewportClip-${plotId})` }, children: annotations }), plotEvents ? ((0, jsx_runtime_1.jsx)(Tracking_1.default, { plotId: plotId, plotEvents: plotEvents, stateSeries: state.series, axisContext: axisContext, plotWidth: plotWidth, plotHeight: plotHeight })) : null] }), (0, jsx_runtime_1.jsx)("g", { transform: `translate(${leftOffset}, ${topOffset})`, children: (0, jsx_runtime_1.jsx)(legendOffsetContext_1.legendOffsetContext.Provider, { value: legendOffset, children: (0, jsx_runtime_1.jsx)("g", { ref: legendBbox.ref, children: legend }) }) })] })] }) }) }) }));
}
exports.Plot = Plot;
